@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: downloader-view.c,v 1.18 2003/06/28 07:36:47 mpeseng Exp $
+ *  $Id: downloader-view.c,v 1.21 2003/07/14 14:22:37 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -325,6 +325,7 @@ controls_info_foreach (GtkTreeModel *model,
 
 	gtk_tree_model_get_value (model, iter, COL_PERSIST_OBJECT, &val);
 	persist_object = g_value_get_pointer (&val);
+	g_value_unset (&val);
 
 	details = g_hash_table_lookup (info->priv->details_hash,
 				       persist_object);
@@ -459,7 +460,7 @@ downloader_view_set_download_info (DownloaderViewPrivate *priv,
 				   GtkTreeIter *iter)
 {
 	gchar buffer[50];
-	GtkTreePath *path;
+	GtkTreePath *path = NULL;
 	GtkTreePath *selected_path = NULL;
 	GtkTreeIter selected_iter;
 	GtkTreeSelection *selection;
@@ -550,6 +551,9 @@ downloader_view_set_download_info (DownloaderViewPrivate *priv,
 	{
 		downloader_view_update_details (priv, details);
 	}
+
+	gtk_tree_path_free (path);
+	gtk_tree_path_free (selected_path);
 }
 
 static void
@@ -593,7 +597,7 @@ downloader_view_add_download (DownloaderView *dv,
 	details->remaining = -1;
 	details->speed = -1;
 	details->size_total = -1;
-	details->size_done = -1;
+	details->size_done = 0;
 	details->progress = -1;
 	details->download_started = FALSE;
 	dv->priv->show_details = FALSE;
@@ -630,15 +634,17 @@ downloader_view_remove_download (DownloaderView *dv,
 				 gpointer persist_object)
 {
 	DownloadDetails *details;
+	GtkTreePath *path = NULL;
 	GtkTreeIter iter;
 
 	details = g_hash_table_lookup (dv->priv->details_hash,
 				       persist_object);
 	g_return_if_fail (details);
 
+	path = gtk_tree_row_reference_get_path (details->ref);
+
 	gtk_tree_model_get_iter (GTK_TREE_MODEL (dv->priv->model),
-				 &iter,
-				 gtk_tree_row_reference_get_path (details->ref));
+				 &iter, path);
 
 	gtk_list_store_remove (GTK_LIST_STORE (dv->priv->model), &iter);
 
@@ -646,6 +652,8 @@ downloader_view_remove_download (DownloaderView *dv,
 			     persist_object);
 
 	ensure_selected_row (dv);
+
+	gtk_tree_path_free (path);
 }
 
 void
@@ -659,6 +667,7 @@ downloader_view_set_download_progress (DownloaderView *dv,
 				       gpointer persist_object)
 {
 	DownloadDetails *details;
+	GtkTreePath *path = NULL;
 	GtkTreeIter iter;
 
 	details = g_hash_table_lookup (dv->priv->details_hash,
@@ -672,11 +681,14 @@ downloader_view_set_download_progress (DownloaderView *dv,
 	details->size_done = size_done;
 	details->progress = progress;
 
+	path = gtk_tree_row_reference_get_path (details->ref);
+
 	gtk_tree_model_get_iter (GTK_TREE_MODEL (dv->priv->model),
-				 &iter,
-				 gtk_tree_row_reference_get_path (details->ref));
+				 &iter, path);
 
 	downloader_view_set_download_info (dv->priv, details, &iter);
+
+	gtk_tree_path_free (path);
 }
 
 void
@@ -685,6 +697,7 @@ downloader_view_set_download_status (DownloaderView *dv,
 				     gpointer persist_object)
 {
 	DownloadDetails *details;
+	GtkTreePath *path = NULL;
 	GtkTreeIter iter;
 
 	details = g_hash_table_lookup (dv->priv->details_hash,
@@ -693,9 +706,10 @@ downloader_view_set_download_status (DownloaderView *dv,
 
 	details->status = status;
 
+	path = gtk_tree_row_reference_get_path (details->ref);
+
 	gtk_tree_model_get_iter (GTK_TREE_MODEL (dv->priv->model),
-				 &iter,
-				 gtk_tree_row_reference_get_path (details->ref));
+				 &iter, path);
 
 	downloader_view_set_download_info (dv->priv, details, &iter);
 	downloader_view_update_controls (dv->priv);
@@ -704,6 +718,8 @@ downloader_view_set_download_status (DownloaderView *dv,
 	{
 		downloader_view_remove_download (dv, persist_object);
 	}*/
+
+	gtk_tree_path_free (path);
 }
 
 static void
@@ -832,6 +848,7 @@ download_dialog_pause_cb (GtkButton *button, DownloaderView *dv)
 
 	gtk_tree_model_get_value (model, &iter, COL_PERSIST_OBJECT, &val);
 	persist_object = g_value_get_pointer (&val);
+	g_value_unset (&val);
 
 	details = g_hash_table_lookup (dv->priv->details_hash,
 				       persist_object);
@@ -872,17 +889,20 @@ download_dialog_abort_cb (GtkButton *button, DownloaderView *dv)
         for (r = rlist; r != NULL; r = r->next)
         {
                 GtkTreeRowReference *node = r->data;
+		GtkTreePath *path = NULL;
 		GValue val = {0, };
 		gpointer *persist_object;
 		GtkTreeIter iter;
 		DownloadDetails *details;
 
-                gtk_tree_model_get_iter (model, &iter,
-                                         gtk_tree_row_reference_get_path (node));
+		path = gtk_tree_row_reference_get_path (node);
+
+                gtk_tree_model_get_iter (model, &iter, path);
 
 		gtk_tree_model_get_value (model, &iter,
 					  COL_PERSIST_OBJECT, &val);
 		persist_object = g_value_get_pointer (&val);
+		g_value_unset (&val);
 
 		details = g_hash_table_lookup (dv->priv->details_hash,
 					       persist_object);
@@ -893,6 +913,7 @@ download_dialog_abort_cb (GtkButton *button, DownloaderView *dv)
 		downloader_view_remove_download (dv, persist_object);
 
                 gtk_tree_row_reference_free (node);
+		gtk_tree_path_free (path);
         }
 
 	g_list_foreach (llist, (GFunc)gtk_tree_path_free, NULL);
@@ -921,6 +942,7 @@ downloader_treeview_selection_changed_cb (GtkTreeSelection *selection,
 	{
 		gtk_tree_model_get_value (priv->model, &iter, COL_PERSIST_OBJECT, &val);
 		persist_object = g_value_get_pointer (&val);
+		g_value_unset (&val);
 
 		details = g_hash_table_lookup (priv->details_hash,
 					       persist_object);
@@ -1041,6 +1063,7 @@ open_selection_foreach (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *ite
 
 	gtk_tree_model_get_value (model, iter, COL_PERSIST_OBJECT, &val);
 	persist_object = g_value_get_pointer (&val);
+	g_value_unset (&val);
 
 	details = g_hash_table_lookup (dv->priv->details_hash,
 				       persist_object);
