@@ -37,7 +37,6 @@
 #include "nsIHistoryEntry.h"
 #include "nsIWebBrowserPrint.h"
 #include "nsIURI.h"
-#include "nsIPresShell.h"
 #include "nsIMarkupDocumentViewer.h"
 #include "nsIComponentManager.h"
 #include "nsIScriptGlobalObject.h"
@@ -66,7 +65,6 @@
 #include "nsIDOMHTMLDocument.h"
 #include "nsIDOMHTMLCollection.h"
 #include "nsIDOMHTMLElement.h"
-#include "caps/nsIPrincipal.h"
 #include "nsIDeviceContext.h"
 #include "nsIPresContext.h"
 #include "ContentHandler.h"
