@@ -35,12 +35,15 @@
 #include <locale.h>
 #include <libgnome/gnome-i18n.h>
 #include <stdlib.h>
-#include <sys/utsname.h>
-#include "nsBuildID.h"
 #include "nsCOMPtr.h"
 #include "nsIPrefService.h"
 #include "nsIServiceManager.h"
 
+static void
+mozilla_cache_size_notifier (GConfClient *client,
+		             guint cnxn_id,
+		             GConfEntry *entry,
+		             char *pref);
 static void
 mozilla_own_colors_notifier(GConfClient *client,
 			    guint cnxn_id,
@@ -79,12 +82,6 @@ mozilla_language_notifier(GConfClient *client,
 			  GConfEntry *entry,
 			  EphyEmbedSingle *single);
 
-static void
-mozilla_default_font_notifier(GConfClient *client,
-			      guint cnxn_id,
-			      GConfEntry *entry,
-			      EphyEmbedSingle *single);
-
 static void
 mozilla_proxy_mode_notifier (GConfClient *client,
 			     guint cnxn_id,
@@ -95,18 +92,16 @@ mozilla_proxy_autoconfig_notifier (GConfClient *client,
 			           guint cnxn_id,
 			           GConfEntry *entry,
 			           EphyEmbedSingle *single);
-
 static void
-mozilla_user_agent_notifier(GConfClient *client,
-			    guint cnxn_id,
-			    GConfEntry *entry,
-			    EphyEmbedSingle *single);
-
+mozilla_proxy_mode_notifier (GConfClient *client,
+		             guint cnxn_id,
+		             GConfEntry *entry,
+		             EphyEmbedSingle *single);
 static void
-mozilla_socks_version_notifier (GConfClient *client,
-				guint cnxn_id,
-				GConfEntry *entry,
-				EphyEmbedSingle *single);
+mozilla_cookies_accept_notifier (GConfClient *client,
+		                 guint cnxn_id,
+		                 GConfEntry *entry,
+		                 char *pref);
 
 /* Keeps the list of the notifiers we installed for mozilla prefs */
 /* to be able to remove them when exiting */
@@ -130,23 +125,17 @@ conversion_table [] =
 {
 	{ CONF_SECURITY_JAVA_ENABLED, BOOL_PREF, "security.enable_java"},
 	{ CONF_SECURITY_JAVASCRIPT_ENABLED, BOOL_PREF, "javascript.enabled"},
-	{ CONF_RENDERING_UNDERLINE_LINKS, BOOL_PREF, "browser.underline_anchors"},
 	{ CONF_NETWORK_PROXY_AUTO_URL, STRING_PREF, "network.proxy.autoconfig_url"},
 	{ CONF_NETWORK_HTTP_PROXY, STRING_PREF, "network.proxy.http"},
 	{ CONF_NETWORK_FTP_PROXY, STRING_PREF, "network.proxy.ftp"},
 	{ CONF_NETWORK_SSL_PROXY, STRING_PREF, "network.proxy.ssl"},
-	{ CONF_NETWORK_SOCKS_PROXY, STRING_PREF, "network.proxy.socks"},
 	{ CONF_NETWORK_HTTP_PROXY_PORT, INT_PREF, "network.proxy.http_port"},
 	{ CONF_NETWORK_FTP_PROXY_PORT, INT_PREF, "network.proxy.ftp_port"},
 	{ CONF_NETWORK_SSL_PROXY_PORT, INT_PREF, "network.proxy.ssl_port"},
-	{ CONF_NETWORK_SOCKS_PROXY_PORT, INT_PREF, "network.proxy.socks_port"},
-	{ CONF_NETWORK_NO_PROXIES_FOR, STRING_PREF, "network.proxy.no_proxies_on"},
-	{ CONF_NETWORK_MEMORY_CACHE, INT_PREF, "browser.cache.memory.capacity"},
-	{ CONF_NETWORK_DISK_CACHE, INT_PREF, "browser.cache.disk.capacity"},
-	{ CONF_NETWORK_CACHE_COMPARE, INT_PREF, "browser.cache.check_doc_frequency"},
-	{ CONF_SECURITY_COOKIES_ACCEPT, BOOL_PREF, "network.cookie.warnAboutCookies"},
 	{ CONF_LANGUAGE_DEFAULT_ENCODING, STRING_PREF, "intl.charset.default" },
 	{ CONF_LANGUAGE_AUTODETECT_ENCODING, STRING_PREF, "intl.charset.detector" },
+	{ CONF_RENDERING_DEFAULT_FONT, STRING_PREF, "font.default" },
+
 	{ NULL, 0, NULL }
 };
  
@@ -157,8 +146,6 @@ static const struct
 }
 custom_notifiers [] =
 {
-	{ CONF_NETWORK_USER_AGENT, 
-	  (GConfClientNotifyFunc) mozilla_user_agent_notifier },
 	{ CONF_RENDERING_USE_OWN_COLORS, 
 	  (GConfClientNotifyFunc) mozilla_own_colors_notifier },
 	{ CONF_RENDERING_USE_OWN_FONTS, 
@@ -167,15 +154,16 @@ custom_notifiers [] =
 	  (GConfClientNotifyFunc) mozilla_allow_popups_notifier },
 	{ CONF_RENDERING_LANGUAGE, 
 	  (GConfClientNotifyFunc) mozilla_language_notifier },
-	{ CONF_RENDERING_DEFAULT_FONT, 
-	  (GConfClientNotifyFunc) mozilla_default_font_notifier },
-	{ CONF_NETWORK_SOCKS_PROXY_VERSION, 
-	  (GConfClientNotifyFunc) mozilla_socks_version_notifier },
 	{ CONF_NETWORK_PROXY_MODE,
 	  (GConfClientNotifyFunc) mozilla_proxy_mode_notifier },
 	{ CONF_NETWORK_PROXY_AUTO_URL,
 	  (GConfClientNotifyFunc) mozilla_proxy_autoconfig_notifier },
-	{NULL, NULL}
+	{ CONF_NETWORK_CACHE_SIZE,
+          (GConfClientNotifyFunc) mozilla_cache_size_notifier },
+	{ CONF_SECURITY_COOKIES_ACCEPT,
+	  (GConfClientNotifyFunc) mozilla_cookies_accept_notifier },
+
+	{ NULL, NULL }
 };
 
 static gboolean
@@ -236,6 +224,19 @@ mozilla_prefs_set_int (const char *preference_name, int new_int_value)
         return FALSE;
 }
 
+static void
+mozilla_cache_size_notifier (GConfClient *client,
+		             guint cnxn_id,
+		             GConfEntry *entry,
+		             char *pref)
+{
+	int cache_size;
+
+	cache_size = gconf_value_get_int(entry->value) * 1024;
+
+	mozilla_prefs_set_int ("browser.cache.disk.capacity", cache_size);
+}
+
 static void
 mozilla_font_size_notifier (GConfClient *client,
 		            guint cnxn_id,
@@ -272,6 +273,33 @@ mozilla_proxy_mode_notifier (GConfClient *client,
 	mozilla_prefs_set_int ("network.proxy.type", mozilla_mode);
 }
 
+static void
+mozilla_cookies_accept_notifier (GConfClient *client,
+		                 guint cnxn_id,
+		                 GConfEntry *entry,
+		                 char *pref)
+{
+	const char *mode;
+	int mozilla_mode = 0;
+	
+	mode = gconf_value_get_string(entry->value);
+	
+	if (strcmp (mode, "anywhere") == 0)
+	{
+		mozilla_mode = 0;
+	}
+	else if (strcmp (mode, "current site") == 0)
+	{
+		mozilla_mode = 1;
+	}
+	else if (strcmp (mode, "nowhere") == 0)
+	{
+		mozilla_mode = 2;
+	}
+
+	mozilla_prefs_set_int ("network.cookie.cookieBehavior", mozilla_mode);
+}
+
 static void
 mozilla_font_notifier (GConfClient *client,
 		       guint cnxn_id,
@@ -337,6 +365,8 @@ mozilla_notifiers_init(EphyEmbedSingle *single)
 {
 	GConfClient *client = eel_gconf_client_get_global ();
 	guint i;
+	guint n_fonts_languages;
+	const FontsLanguageInfo *fonts_language;
 
 	for (i = 0; conversion_table[i].gconf_key != NULL; i++)
 	{
@@ -374,6 +404,8 @@ mozilla_notifiers_init(EphyEmbedSingle *single)
 	}
 
 	/* fonts notifiers */
+	n_fonts_languages = ephy_langs_get_n_font_languages ();
+	fonts_language = ephy_langs_get_font_languages ();
 	for (i = 0; i < n_fonts_languages; i++)
 	{
 		guint k;
@@ -385,30 +417,30 @@ mozilla_notifiers_init(EphyEmbedSingle *single)
 		{
 			info = g_strconcat (types[k], ".", fonts_language[i].code, NULL);
 			
-			snprintf (key, 255, "%s_%s_%s", CONF_RENDERING_FONT, 
-			 	 types[k], 
-                 	 	 fonts_language[i].code);
+			g_snprintf (key, 255, "%s_%s_%s", CONF_RENDERING_FONT, 
+			 	    types[k], 
+                 	 	    fonts_language[i].code);
 			add_notification_and_notify (client, key,
 						     (GConfClientNotifyFunc)mozilla_font_notifier,
 						     info);
 			font_infos = g_list_append (font_infos, info);
 		}
 
-		snprintf (key, 255, "%s_%s", CONF_RENDERING_FONT_MIN_SIZE, fonts_language[i].code);
+		g_snprintf (key, 255, "%s_%s", CONF_RENDERING_FONT_MIN_SIZE, fonts_language[i].code);
 		info = g_strconcat ("minimum-size", ".", fonts_language[i].code, NULL);
 		add_notification_and_notify (client, key,
 					     (GConfClientNotifyFunc)mozilla_font_size_notifier,
 					     info);
 		font_infos = g_list_append (font_infos, info);
 
-		snprintf (key, 255, "%s_%s", CONF_RENDERING_FONT_FIXED_SIZE, fonts_language[i].code);
+		g_snprintf (key, 255, "%s_%s", CONF_RENDERING_FONT_FIXED_SIZE, fonts_language[i].code);
 		info = g_strconcat ("size.fixed", ".", fonts_language[i].code, NULL);
 		add_notification_and_notify (client, key,
 					     (GConfClientNotifyFunc)mozilla_font_size_notifier,
 					     info);
 		font_infos = g_list_append (font_infos, info);
 
-		snprintf (key, 255, "%s_%s", CONF_RENDERING_FONT_VAR_SIZE, fonts_language[i].code);
+		g_snprintf (key, 255, "%s_%s", CONF_RENDERING_FONT_VAR_SIZE, fonts_language[i].code);
 		info = g_strconcat ("size.variable", ".", fonts_language[i].code, NULL);
 		add_notification_and_notify (client, key,
 					     (GConfClientNotifyFunc)mozilla_font_size_notifier,
@@ -529,8 +561,10 @@ mozilla_own_fonts_notifier(GConfClient *client,
 			   GConfEntry *entry,
 			   EphyEmbedSingle *single)
 {
-	mozilla_prefs_set_int("browser.display.use_document_fonts",
-				   !gconf_value_get_bool(entry->value));
+	int value;
+
+	value = gconf_value_get_bool(entry->value) ? 0 : 1;
+	mozilla_prefs_set_int("browser.display.use_document_fonts", value);
 }
 
 static void
@@ -548,7 +582,21 @@ static char *
 get_system_language ()
 {
 	const GList *sys_langs;
+	const char *lang;
 
+	/**
+	 * translate this as the comma separated list of language ranges
+	 * for your locale, as specified by RFC 2616, 14.4. 
+	 * for example for en_NZ locale this could be "en-nz,en-au,en-gb,en"
+	 */
+	lang = _("system-language");
+	
+	if (strncmp (lang, "system-language", 15) != 0)
+	{
+		/* the l10n has it */
+		return g_strdup (lang);
+	}
+	
 	sys_langs = gnome_i18n_get_language_list ("LC_MESSAGES");
 
 	if (sys_langs)
@@ -609,103 +657,3 @@ mozilla_language_notifier(GConfClient *client,
 	g_slist_foreach (languages, (GFunc) g_free, NULL);
 	g_slist_free (languages);
 }
-
-static void
-mozilla_default_font_notifier(GConfClient *client,
-			      guint cnxn_id,
-			      GConfEntry *entry,
-			      EphyEmbedSingle *single)
-{
-	const gchar *font_types [] = {"serif","sans-serif"};
-	int default_font;
-
-	default_font = eel_gconf_get_integer (CONF_RENDERING_DEFAULT_FONT);
-	if (default_font < 0 || 
-	    default_font >= (int)(sizeof(font_types) / sizeof(font_types[0])))
-	{
-		g_warning ("mozilla_default_font_notifier: "
-			   "unsupported value: %d", default_font);
-		return;
-	}
-	mozilla_prefs_set_string ("font.default", font_types[default_font]);
-}
-
-static void
-mozilla_prefs_set_user_agent ()
-{
-        static gchar *default_user_agent = NULL;
-        gchar *value;
-        gchar *user_agent = NULL;
-        struct utsname name;
-        gchar *system;
-
-        if (!default_user_agent)
-        {
-                if (uname (&name) == 0)
-                {
-                        system = g_strdup_printf ("%s %s",
-                                                  name.sysname, 
-                                                  name.machine);
-                }
-                else
-                {
-                        system = g_strdup ("Unknown");
-                }
-                
-                default_user_agent = g_strdup_printf
-                        ("Mozilla/5.0 (X11; U; %s) Gecko/%d Epiphany/" VERSION, 
-                         system,
-                         NS_BUILD_ID/100);
-
-                g_free (system);
-        }
-
-        value = eel_gconf_get_string (CONF_NETWORK_USER_AGENT);
-        
-        /* now, build a valid user agent string */
-        if (!value || !strcmp ("", value) 
-                   || !strcmp ("default", value) 
-                   || !strcmp ("Ephy", value)
-                   || !strcmp (_("Default (recommended)"), value)
-                   || !strcmp ("Default (recommended)", value))
-        {
-                user_agent = g_strdup (default_user_agent);
-        }
-        else
-        {
-                user_agent = g_strdup (value);
-        }
-
-	mozilla_prefs_set_string ("general.useragent.override", user_agent);
-	g_free (user_agent);
-}
-
-static void
-mozilla_user_agent_notifier (GConfClient *client,
-			     guint cnxn_id,
-			     GConfEntry *entry,
-			     EphyEmbedSingle *single)
-{
-	switch (entry->value->type)
-	{
-		case GCONF_VALUE_STRING:
-			mozilla_prefs_set_user_agent ();
-			break;
-
-		default:	
-			g_warning ("Unsupported variable type");
-			break;
-	}
-}
-
-static void
-mozilla_socks_version_notifier (GConfClient *client,
-				guint cnxn_id,
-				GConfEntry *entry,
-				EphyEmbedSingle *single)
-{
-	int version;
-	version = gconf_value_get_int(entry->value) + 4;
-	mozilla_prefs_set_int ("network.proxy.socks_version", 
-			       version);
-}
