@@ -15,6 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
+ *  $Id: ephy-node-view.c,v 1.16 2003/07/14 09:51:37 xan Exp $
  */
 
 #include <gtk/gtktreeselection.h>
@@ -66,6 +67,8 @@ struct EphyNodeViewPrivate
 
 	gboolean editing;
 	int editable_property;
+
+	int searchable_data_column;
 };
 
 enum
@@ -260,6 +263,8 @@ drag_motion_cb (GtkWidget *widget,
 		gdk_drag_status (context, 0, time);
 	}
 
+	gtk_tree_path_free (path);
+
 	return TRUE;
 }
 
@@ -318,6 +323,7 @@ drag_data_received_cb (GtkWidget *widget,
 
 		g_list_free (src_nodes);
 
+		gtk_tree_path_free (path);
 	}
 
 	return TRUE;
@@ -414,6 +420,57 @@ ephy_node_view_row_activated_cb (GtkTreeView *treeview,
 	g_signal_emit (G_OBJECT (view), ephy_node_view_signals[NODE_ACTIVATED], 0, node);
 }
 
+
+static gboolean
+ephy_node_view_select_node_by_key (EphyNodeView *view, GdkEventKey *event)
+{
+	GtkTreeIter iter, last_iter;
+	GtkTreePath *path;
+	GValue value = {0, };
+	gchar *string;
+	gchar *event_string;
+	gboolean found = FALSE;
+	gchar outbuf[6];
+	gint length;
+	
+	length = g_unichar_to_utf8 (gdk_keyval_to_unicode (event->keyval), outbuf);
+	event_string = g_utf8_casefold (outbuf, length);
+
+	if (!gtk_tree_model_get_iter_first (view->priv->sortmodel, &iter))
+	{
+		g_free (event_string);
+		return FALSE;
+	}
+
+	do
+	{
+		last_iter = iter;
+		gtk_tree_model_get_value (view->priv->sortmodel, &iter,
+					  view->priv->searchable_data_column,
+					  &value);
+
+		string = g_utf8_casefold (g_value_get_string (&value), -1);
+		g_utf8_strncpy (string, string, 1);
+		found = (g_utf8_collate (string, event_string) >= 0);
+
+		g_free (string);
+		g_value_unset (&value);
+	}
+	while (!found && gtk_tree_model_iter_next (view->priv->sortmodel, &iter));
+
+	if (!found)
+	{
+		iter = last_iter;
+	}
+	
+	path = gtk_tree_model_get_path (view->priv->sortmodel, &iter);
+	gtk_tree_view_set_cursor (GTK_TREE_VIEW (view), path, NULL, FALSE);
+	gtk_tree_path_free (path);
+	g_free (event_string);
+
+	return TRUE;
+}
+
 static gboolean
 ephy_node_view_key_press_cb (GtkTreeView *treeview,
 			     GdkEventKey *event,
@@ -426,6 +483,11 @@ ephy_node_view_key_press_cb (GtkTreeView *treeview,
 
 		return TRUE;
 	}
+	else if (view->priv->searchable_data_column != -1 &&
+		 gdk_keyval_to_unicode (event->keyval))
+	{
+		return ephy_node_view_select_node_by_key (view, event);
+	}
 
 	return FALSE;
 }
@@ -435,7 +497,7 @@ ephy_node_view_button_press_cb (GtkTreeView *treeview,
 			        GdkEventButton *event,
 			        EphyNodeView *view)
 {
-	GtkTreePath *path;
+	GtkTreePath *path = NULL;
 	GtkTreeSelection *selection;
 	gboolean result = FALSE;
 
@@ -464,6 +526,7 @@ ephy_node_view_button_press_cb (GtkTreeView *treeview,
 			target_view = view;
 			g_signal_emit (G_OBJECT (view), ephy_node_view_signals[SHOW_POPUP], 0);
 			target_view = NULL;
+			gtk_tree_path_free (path);
 		}
 	}
 
@@ -562,11 +625,10 @@ ephy_node_view_construct (EphyNodeView *view)
 			         G_CALLBACK (ephy_node_view_button_press_cb),
 			         view,
 				 0);
-	g_signal_connect_object (G_OBJECT (view),
+	g_signal_connect_after (G_OBJECT (view),
 			         "key_press_event",
 			         G_CALLBACK (ephy_node_view_key_press_cb),
-			         view,
-				 0);
+			         view);
 	g_signal_connect_object (G_OBJECT (view),
 			         "row_activated",
 			         G_CALLBACK (ephy_node_view_row_activated_cb),
@@ -661,9 +723,10 @@ ephy_node_view_sort_func (GtkTreeModel *model,
 		switch (G_TYPE_FUNDAMENTAL (type))
 		{
 		case G_TYPE_STRING:
+			/* FIXME: this is horribly inefficient */
 			stra = g_utf8_casefold (g_value_get_string (&a_value), -1);
 			strb = g_utf8_casefold (g_value_get_string (&b_value), -1);
-			g_return_val_if_fail (stra != NULL || strb != NULL, 0);
+			g_return_val_if_fail (stra != NULL && strb != NULL, 0);
 			retval = g_utf8_collate (stra, strb);
 			g_free (stra);
 			g_free (strb);
@@ -776,6 +839,12 @@ ephy_node_view_add_column (EphyNodeView *view,
 		g_signal_connect (renderer, "edited", G_CALLBACK (cell_renderer_edited), view);
 	}
 
+	if ((flags & EPHY_NODE_VIEW_SEARCHABLE) &&
+	    (view->priv->searchable_data_column == -1))
+	{
+		view->priv->searchable_data_column = column;
+	}
+
 	gtk_tree_view_column_pack_start (gcolumn, renderer, TRUE);
 	gtk_tree_view_column_set_attributes (gcolumn, renderer,
 					     "text", column,
@@ -845,6 +914,7 @@ ephy_node_view_init (EphyNodeView *view)
 	view->priv->editable_renderer = NULL;
 	view->priv->editing = TRUE;
 	view->priv->selected_node = NULL;
+	view->priv->searchable_data_column = -1;
 
 	gtk_tree_view_set_enable_search (GTK_TREE_VIEW (view), FALSE);
 }
@@ -906,15 +976,16 @@ ephy_node_view_remove (EphyNodeView *view)
 	EphyNode *node;
 	GtkTreeIter iter, iter2;
 	GtkTreePath *path;
+	GtkTreeRowReference *row_ref = NULL;
 
-	/* Before removing we try to select the next node in the view. If that is
+	/* Before removing we try to get a reference to the next node in the view. If that is
 	 * not available we try with the previous one, and if that is absent too,
-	 * we do not select anything (which equals to select the topic "All")
+	 * we will not select anything (which equals to select the topic "All")
 	 */
 
 	list = ephy_node_view_get_selection (view);
 	g_return_if_fail (list != NULL);
-	node = g_list_last (list)->data;
+	node = g_list_first (list)->data;
 	ephy_tree_model_node_iter_from_node (EPHY_TREE_MODEL_NODE (view->priv->nodemodel),
 					     node, &iter);
 	egg_tree_model_filter_convert_child_iter_to_iter (EGG_TREE_MODEL_FILTER (view->priv->filtermodel),
@@ -926,18 +997,17 @@ ephy_node_view_remove (EphyNodeView *view)
 	if (gtk_tree_model_iter_next (GTK_TREE_MODEL (view->priv->sortmodel), &iter))
 	{
 		path = gtk_tree_model_get_path (GTK_TREE_MODEL (view->priv->sortmodel), &iter);
-		gtk_tree_view_set_cursor (GTK_TREE_VIEW (view), path, NULL, FALSE);
-		gtk_tree_path_free (path);
+		row_ref = gtk_tree_row_reference_new (GTK_TREE_MODEL (view->priv->sortmodel), path);
 	}
 	else
 	{
 		path = gtk_tree_model_get_path (GTK_TREE_MODEL (view->priv->sortmodel), &iter2);
 		if (gtk_tree_path_prev (path))
 		{
-			gtk_tree_view_set_cursor (GTK_TREE_VIEW (view), path, NULL, FALSE);
+			row_ref = gtk_tree_row_reference_new (GTK_TREE_MODEL (view->priv->sortmodel), path);
 		}
-		gtk_tree_path_free (path);
 	}
+	gtk_tree_path_free (path);
 
 	for (; list != NULL; list = list->next)
 	{
@@ -945,6 +1015,16 @@ ephy_node_view_remove (EphyNodeView *view)
 	}
 
 	g_list_free (list);
+
+	/* Select the "next" node */
+
+	if (row_ref != NULL)
+	{
+		path = gtk_tree_row_reference_get_path (row_ref);
+		gtk_tree_view_set_cursor (GTK_TREE_VIEW (view), path, NULL, FALSE);
+		gtk_tree_row_reference_free (row_ref);
+		gtk_tree_path_free (path);
+	}
 }
 
 void
@@ -1068,3 +1148,4 @@ ephy_node_view_has_selection (EphyNodeView *view, gboolean *multiple)
 
 	return rows > 0;
 }
+
