@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmarks.c,v 1.48 2003/06/04 15:27:13 mpeseng Exp $
+ *  $Id: ephy-bookmarks.c,v 1.56 2003/07/13 08:52:36 mpeseng Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -31,20 +31,26 @@
 #include "ephy-node-common.h"
 #include "ephy-toolbars-model.h"
 #include "ephy-bookmarks-export.h"
+#include "ephy-bookmarks-import.h"
+#include "ephy-autocompletion.h"
+#include "session.h"
 
 #include <string.h>
 #include <bonobo/bonobo-i18n.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 
-#define EPHY_BOOKMARKS_XML_VERSION "0.1"
-
+#define EPHY_BOOKMARKS_XML_VERSION "1.0"
+#define BOOKMARKS_SAVE_DELAY (3 * 1000)
 #define MAX_FAVORITES_NUM 10
 
 struct EphyBookmarksPrivate
 {
 	EphyToolbarsModel *toolbars_model;
 	gboolean init_defaults;
+	gboolean dirty;
+	guint save_timeout_id;
 	char *xml_file;
+	char *rdf_file;
 	EphyNodeDb *db;
 	EphyNode *bookmarks;
 	EphyNode *keywords;
@@ -68,7 +74,7 @@ static const EphyBookmarksBookmarkInfo default_bookmarks [] =
 	 * "http://www.google.nl" and "http://www.google.nl/search?q=%s"
 	 */
 
-	{ N_("Search the web"), N_("http://www.google.com"), N_("http://www.google.com/search?q=%s&ie=UTF-8&oe=UTF-8") }
+	{ N_("Search the web"), N_("http://www.google.com/search?q=%s&ie=UTF-8&oe=UTF-8") }
 };
 static int n_default_bookmarks = G_N_ELEMENTS (default_bookmarks);
 
@@ -169,30 +175,21 @@ ephy_bookmarks_autocompletion_source_foreach (EphyAutocompletionSource *source,
 	for (i = 0; i < children->len; i++)
 	{
 		EphyNode *kid;
-		const char *url, *smart_url, *title, *keywords;
+		const char *url, *title, *keywords;
+		gboolean smart_url;
 
 		kid = g_ptr_array_index (children, i);
 		url = ephy_node_get_property_string
 			(kid, EPHY_NODE_BMK_PROP_LOCATION);
-		smart_url = ephy_node_get_property_string
-			(kid, EPHY_NODE_BMK_PROP_SMART_LOCATION);
+		smart_url = ephy_node_get_property_boolean
+			(kid, EPHY_NODE_BMK_PROP_HAS_SMART_ADDRESS);
 		title = ephy_node_get_property_string
 			(kid, EPHY_NODE_BMK_PROP_TITLE);
 		keywords = ephy_node_get_property_string
 			(kid, EPHY_NODE_BMK_PROP_KEYWORDS);
 
-		if (smart_url == NULL ||
-		    g_utf8_strlen (smart_url, -1) == 0)
-		{
-			smart_url = NULL;
-		}
-
-		func (source,
-		      smart_url ? NULL : keywords,
-		      title,
-		      smart_url ? smart_url : url,
-		      (smart_url != NULL),
-		      TRUE, 0, data);
+		func (source, keywords, title, url, smart_url,
+		      !smart_url, 0, data);
 	}
 	ephy_node_thaw (eb->priv->bookmarks);
 }
@@ -224,16 +221,13 @@ ephy_bookmarks_init_defaults (EphyBookmarks *eb)
 	for (i = 0; i < n_default_bookmarks; i++)
 	{
 		ephy_bookmarks_add (eb, default_bookmarks[i].title,
-				    default_bookmarks[i].location,
-				    default_bookmarks[i].smart_url);
+				    default_bookmarks[i].location);
 
 		node = ephy_bookmarks_find_bookmark (eb, default_bookmarks[i].location);
 		if (node == NULL) break;
 		id = ephy_node_get_id (node);
 		ephy_toolbars_model_add_bookmark (eb->priv->toolbars_model, FALSE, id);
 	}
-
-	ephy_bookmarks_save (eb);
 }
 
 static void
@@ -323,6 +317,7 @@ ephy_bookmarks_load (EphyBookmarks *eb)
 	xmlDocPtr doc;
 	xmlNodePtr root, child;
 	char *tmp;
+	gboolean result = TRUE;
 
 	if (g_file_test (eb->priv->xml_file, G_FILE_TEST_EXISTS) == FALSE)
 		return FALSE;
@@ -333,7 +328,12 @@ ephy_bookmarks_load (EphyBookmarks *eb)
 	root = xmlDocGetRootElement (doc);
 
 	tmp = xmlGetProp (root, "version");
-	g_assert (tmp != NULL && strcmp (tmp, EPHY_BOOKMARKS_XML_VERSION) == 0);
+	if (tmp != NULL && strcmp (tmp, EPHY_BOOKMARKS_XML_VERSION) != 0)
+	{
+		g_warning ("Old bookmarks database format detected");
+		child = NULL;
+		result = FALSE;
+	}
 	g_free (tmp);
 
 	for (child = root->children; child != NULL; child = child->next)
@@ -345,17 +345,17 @@ ephy_bookmarks_load (EphyBookmarks *eb)
 
 	xmlFreeDoc (doc);
 
-	return TRUE;
+	return result;
 }
 
-void
+static void
 ephy_bookmarks_save (EphyBookmarks *eb)
 {
 	xmlDocPtr doc;
 	xmlNodePtr root;
+	xmlNodePtr comment;
 	GPtrArray *children;
 	int i;
-	char *rdf_file;
 
 	LOG ("Saving bookmarks")
 
@@ -367,6 +367,9 @@ ephy_bookmarks_save (EphyBookmarks *eb)
 	xmlSetProp (root, "version", EPHY_BOOKMARKS_XML_VERSION);
 	xmlDocSetRootElement (doc, root);
 
+	comment = xmlNewDocComment (doc, "Do not rely on this file, it's only for internal use. Use bookmarks.rdf instead.");
+	xmlAddChild (root, comment);
+
 	children = ephy_node_get_children (eb->priv->keywords);
 	for (i = 0; i < children->len; i++)
 	{
@@ -398,11 +401,40 @@ ephy_bookmarks_save (EphyBookmarks *eb)
 	xmlFreeDoc(doc);
 
 	/* Export bookmarks in rdf */
-	rdf_file = g_build_filename (ephy_dot_dir (),
-				     "bookmarks.rdf",
-				     NULL);
-	ephy_bookmarks_export_rdf (eb, rdf_file);
-	g_free (rdf_file);
+	ephy_bookmarks_export_rdf (eb, eb->priv->rdf_file);
+}
+
+static gboolean
+save_bookmarks_delayed (EphyBookmarks *bookmarks)
+{
+	ephy_bookmarks_save (bookmarks);
+	bookmarks->priv->dirty = FALSE;
+	bookmarks->priv->save_timeout_id = 0;
+
+	return FALSE;
+}
+
+static void
+ephy_bookmarks_save_delayed (EphyBookmarks *bookmarks, int delay)
+{
+	if (!bookmarks->priv->dirty)
+	{
+		bookmarks->priv->dirty = TRUE;
+
+		if (delay > 0)
+		{
+			bookmarks->priv->save_timeout_id =
+				g_timeout_add (BOOKMARKS_SAVE_DELAY,
+				               (GSourceFunc) save_bookmarks_delayed,
+					       bookmarks);
+		}
+		else
+		{
+			bookmarks->priv->save_timeout_id =
+				g_idle_add ((GSourceFunc) save_bookmarks_delayed,
+					    bookmarks);
+		}
+	}
 }
 
 static double
@@ -488,7 +520,7 @@ update_favorites_menus ()
 	Session *session;
 	const GList *l;
 
-	session = ephy_shell_get_session (ephy_shell);
+	session = SESSION (ephy_shell_get_session (ephy_shell));
 	l = session_get_windows (session);
 
 	for (; l != NULL; l = l->next)
@@ -532,6 +564,7 @@ bookmarks_changed_cb (EphyNode *node,
 		      EphyBookmarks *eb)
 {
 	ephy_bookmarks_emit_data_changed (eb);
+	ephy_bookmarks_save_delayed (eb, BOOKMARKS_SAVE_DELAY);
 }
 
 static void
@@ -542,6 +575,7 @@ bookmarks_removed_cb (EphyNode *node,
 {
 	ephy_bookmarks_emit_data_changed (eb);
 	g_signal_emit (G_OBJECT (eb), ephy_bookmarks_signals[TREE_CHANGED], 0);
+	ephy_bookmarks_save_delayed (eb, BOOKMARKS_SAVE_DELAY);
 }
 
 static char *
@@ -584,6 +618,7 @@ static void
 update_topics_list (EphyNode *bookmark, const char *list)
 {
 	GValue value = { 0, };
+
 	g_value_init (&value, G_TYPE_STRING);
 	g_value_set_string (&value, list);
 	ephy_node_set_property (bookmark, EPHY_NODE_BMK_PROP_KEYWORDS,
@@ -636,11 +671,15 @@ ephy_bookmarks_init (EphyBookmarks *eb)
         eb->priv = g_new0 (EphyBookmarksPrivate, 1);
 	eb->priv->toolbars_model = NULL;
 
-	db = ephy_node_db_new ("EphyBookmarks");
+	db = ephy_node_db_new (EPHY_NODE_DB_BOOKMARKS);
 	eb->priv->db = db;
-
+	eb->priv->dirty = FALSE;
+	eb->priv->save_timeout_id = 0;
 	eb->priv->xml_file = g_build_filename (ephy_dot_dir (),
-					       "bookmarks.xml",
+					       "ephy-bookmarks.xml",
+					       NULL);
+	eb->priv->rdf_file = g_build_filename (ephy_dot_dir (),
+					       "bookmarks.rdf",
 					       NULL);
 
 	/* Bookmarks */
@@ -712,7 +751,13 @@ ephy_bookmarks_init (EphyBookmarks *eb)
 	g_value_unset (&value);
 	ephy_node_add_child (eb->priv->keywords, eb->priv->notcategorized);
 
-	eb->priv->init_defaults = !ephy_bookmarks_load (eb);
+	if (!ephy_bookmarks_load (eb))
+	{
+		if (!ephy_bookmarks_import_rdf (eb, eb->priv->rdf_file))
+		{
+			eb->priv->init_defaults = !ephy_bookmarks_load (eb);
+		}
+	}
 
 	ephy_bookmarks_emit_data_changed (eb);
 
@@ -731,6 +776,11 @@ ephy_bookmarks_finalize (GObject *object)
 
         g_return_if_fail (eb->priv != NULL);
 
+	if (eb->priv->save_timeout_id != 0)
+	{
+		g_source_remove (eb->priv->save_timeout_id);
+	}
+
 	ephy_bookmarks_save (eb);
 
 	ephy_node_unref (eb->priv->bookmarks);
@@ -746,6 +796,9 @@ ephy_bookmarks_finalize (GObject *object)
 					      (gpointer *)&eb->priv->toolbars_model);
 	}
 
+	g_free (eb->priv->xml_file);
+	g_free (eb->priv->rdf_file);
+
         g_free (eb->priv);
 
 	LOG ("Bookmarks finalized")
@@ -763,11 +816,28 @@ ephy_bookmarks_new ()
 	return tab;
 }
 
+static void
+update_has_smart_address (EphyNode *bmk, const char *address)
+{
+	gboolean smart = FALSE;
+	GValue value = { 0, };
+
+	if (address)
+	{
+		smart = strstr (address, "%s") != NULL;
+	}
+
+	g_value_init (&value, G_TYPE_BOOLEAN);
+	g_value_set_boolean (&value, smart);
+	ephy_node_set_property (bmk, EPHY_NODE_BMK_PROP_HAS_SMART_ADDRESS,
+			        &value);
+	g_value_unset (&value);
+}
+
 EphyNode *
 ephy_bookmarks_add (EphyBookmarks *eb,
 		    const char *title,
-		    const char *url,
-		    const char *smart_url)
+		    const char *url)
 {
 	EphyNode *bm;
 	GValue value = { 0, };
@@ -786,21 +856,34 @@ ephy_bookmarks_add (EphyBookmarks *eb,
 			        &value);
 	g_value_unset (&value);
 
-	g_value_init (&value, G_TYPE_STRING);
-	g_value_set_string (&value, smart_url);
-	ephy_node_set_property (bm, EPHY_NODE_BMK_PROP_SMART_LOCATION,
-			        &value);
-	g_value_unset (&value);
+	update_has_smart_address (bm, url);
 
 	ephy_node_add_child (eb->priv->bookmarks, bm);
 	ephy_node_add_child (eb->priv->notcategorized, bm);
 
 	ephy_bookmarks_emit_data_changed (eb);
 	g_signal_emit (G_OBJECT (eb), ephy_bookmarks_signals[TREE_CHANGED], 0);
+	ephy_bookmarks_save_delayed (eb, 0);
 
 	return bm;
 }
 
+void
+ephy_bookmarks_set_address (EphyBookmarks *eb,
+			    EphyNode *bookmark,
+			    const char *address)
+{
+	GValue value = { 0, };
+
+	g_value_init (&value, G_TYPE_STRING);
+	g_value_set_string (&value, address);
+	ephy_node_set_property (bookmark, EPHY_NODE_BMK_PROP_LOCATION,
+			        &value);
+	g_value_unset (&value);
+
+	update_has_smart_address (bookmark, address);
+}
+
 EphyNode*
 ephy_bookmarks_find_bookmark (EphyBookmarks *eb,
 			      const char *url)
@@ -808,6 +891,8 @@ ephy_bookmarks_find_bookmark (EphyBookmarks *eb,
 	GPtrArray *children;
 	int i;
 
+	g_return_val_if_fail (url != NULL, NULL);
+
 	children = ephy_node_get_children (eb->priv->bookmarks);
 	for (i = 0; i < children->len; i++)
 	{
@@ -817,7 +902,7 @@ ephy_bookmarks_find_bookmark (EphyBookmarks *eb,
 		kid = g_ptr_array_index (children, i);
 		location = ephy_node_get_property_string
 			(kid, EPHY_NODE_BMK_PROP_LOCATION);
-		if (strcmp (url, location) == 0)
+		if (location != NULL && strcmp (url, location) == 0)
 		{
 			ephy_node_thaw (eb->priv->bookmarks);
 			return kid;
