@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-topic-action.c,v 1.18 2003/06/14 11:15:35 chpe Exp $
+ *  $Id: ephy-topic-action.c,v 1.21 2003/07/13 08:52:36 mpeseng Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -25,10 +25,12 @@
 #include "ephy-node-common.h"
 #include "ephy-topic-action.h"
 #include "ephy-bookmarks.h"
+#include "ephy-favicon-cache.h"
 #include "ephy-shell.h"
 #include "eggtoolitem.h"
 #include "ephy-debug.h"
 #include "ephy-gui.h"
+#include "ephy-string.h"
 
 static void ephy_topic_action_init       (EphyTopicAction *action);
 static void ephy_topic_action_class_init (EphyTopicActionClass *class);
@@ -141,14 +143,30 @@ menu_activate_cb (GtkWidget *item, EggAction *action)
 static void
 ephy_topic_action_sync_label (EggAction *action, GParamSpec *pspec, GtkWidget *proxy)
 {
-	GtkLabel *label;
+	GtkWidget *label = NULL;
 
 	LOG ("Set bookmark action proxy label to %s", action->label)
 
-	label = GTK_LABEL (g_object_get_data (G_OBJECT (proxy), "label"));
+	if (EGG_IS_TOOL_ITEM (proxy))
+	{
+		label = GTK_WIDGET (g_object_get_data (G_OBJECT (proxy), "label"));
+	}
+	else if (GTK_IS_MENU_ITEM (proxy))
+	{
+		label = GTK_BIN (proxy)->child;
+	}
+	else
+	{
+		g_warning ("Unknown widget");
+		return;
+	}
+
 	g_return_if_fail (label != NULL);
 
-	gtk_label_set_label (label, action->label);
+	if (action->label)
+	{
+		gtk_label_set_label (GTK_LABEL (label), action->label);
+	}
 }
 
 static int
@@ -171,6 +189,8 @@ sort_bookmarks (gconstpointer a, gconstpointer b)
 	return retval;
 }
 
+#define MAX_LENGTH 32
+
 static GtkWidget *
 build_bookmarks_menu (EphyTopicAction *action, EphyNode *node)
 {
@@ -180,8 +200,8 @@ build_bookmarks_menu (EphyTopicAction *action, EphyNode *node)
 
 	menu = gtk_menu_new ();
 
-        cache = ephy_embed_shell_get_favicon_cache
-               (EPHY_EMBED_SHELL (ephy_shell));
+        cache = EPHY_FAVICON_CACHE
+		(ephy_embed_shell_get_favicon_cache (EPHY_EMBED_SHELL (ephy_shell)));
 
 	children = ephy_node_get_children (node);
 
@@ -210,6 +230,7 @@ build_bookmarks_menu (EphyTopicAction *action, EphyNode *node)
 			EphyNode *kid;
                 	const char *icon_location;
 			const char *title;
+			char *title_short;
 	
 			kid = (EphyNode*)l->data;
 	
@@ -218,9 +239,10 @@ build_bookmarks_menu (EphyTopicAction *action, EphyNode *node)
 			title = ephy_node_get_property_string
 				(kid, EPHY_NODE_BMK_PROP_TITLE);
 			if (title == NULL) continue;
-			LOG ("Create menu for bookmark %s", title)
+			title_short = ephy_string_shorten (title, MAX_LENGTH);
+			LOG ("Create menu for bookmark %s", title_short)
 	
-			item = gtk_image_menu_item_new_with_label (title);
+			item = gtk_image_menu_item_new_with_label (title_short);
 			if (icon_location)
 			{
 				GdkPixbuf *icon;
@@ -242,6 +264,8 @@ build_bookmarks_menu (EphyTopicAction *action, EphyNode *node)
 					  G_CALLBACK (menu_activate_cb), action);
 			gtk_widget_show (item);
 			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+			g_free (title_short);
 		}
 
 		g_list_free (node_list);
@@ -403,24 +427,70 @@ button_pressed_cb (GtkWidget *button,
 	 gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
 }
 
+static GtkWidget *
+create_menu_item (EggAction *action)
+{
+	GtkWidget *menu, *menu_item;
+
+	LOG ("create_menu_item action %p", action)
+
+	menu_item = gtk_menu_item_new_with_label (action->label);
+
+	menu = build_menu (EPHY_TOPIC_ACTION (action));
+	gtk_widget_show (menu);
+
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), menu);
+
+	return menu_item;
+}
+
+static gboolean
+create_menu_proxy (EggToolItem *item, EggAction *action)
+{
+	GtkWidget *menu_item;
+	char *menu_id;
+
+	LOG ("create_menu_proxy item %p, action %p", item, action)
+
+	menu_item = create_menu_item (action);
+
+	menu_id = g_strdup_printf ("ephy-topic-action-%d-menu-id",
+				   EPHY_TOPIC_ACTION (action)->priv->topic_id);
+
+	egg_tool_item_set_proxy_menu_item (item, menu_id, menu_item);
+
+	g_free (menu_id);
+
+	return TRUE;
+}
+
 static void
 connect_proxy (EggAction *action, GtkWidget *proxy)
 {
 	GtkWidget *button;
 
+	LOG ("connect_proxy action %p, proxy %p", action, proxy)
+
 	(* EGG_ACTION_CLASS (parent_class)->connect_proxy) (action, proxy);
 
 	ephy_topic_action_sync_label (action, NULL, proxy);
 	g_signal_connect_object (action, "notify::label",
 			         G_CALLBACK (ephy_topic_action_sync_label), proxy, 0);
 
-	button = GTK_WIDGET (g_object_get_data (G_OBJECT (proxy), "button"));
-	g_signal_connect (button, "toggled",
-			  G_CALLBACK (button_toggled_cb), action);
+	if (EGG_IS_TOOL_ITEM (proxy))
+	{
+		g_signal_connect_object (proxy, "create_menu_proxy",
+					 G_CALLBACK (create_menu_proxy),
+					 action, 0);
 
-	/* We want the menu to popup up on mouse down */
-	g_signal_connect (button, "pressed",
-			  G_CALLBACK (button_pressed_cb), action);
+		button = GTK_WIDGET (g_object_get_data (G_OBJECT (proxy), "button"));
+		g_signal_connect (button, "toggled",
+				  G_CALLBACK (button_toggled_cb), action);
+
+		/* We want the menu to popup up on mouse down */
+		g_signal_connect (button, "pressed",
+				  G_CALLBACK (button_pressed_cb), action);
+	}
 }
 
 static void
@@ -488,6 +558,7 @@ ephy_topic_action_class_init (EphyTopicActionClass *class)
 
 	action_class->toolbar_item_type = EGG_TYPE_TOOL_ITEM;
 	action_class->create_tool_item = create_tool_item;
+	action_class->create_menu_item = create_menu_item;
 	action_class->connect_proxy = connect_proxy;
 
 	object_class->finalize = ephy_topic_action_finalize;
