@@ -50,6 +50,8 @@
 #include "toolbar.h"
 #include "ephy-stock-icons.h"
 #include "ephy-search-entry.h"
+#include "session.h"
+#include "ephy-favicon-cache.h"
 
 static GtkTargetEntry page_drag_types [] =
 {
@@ -193,7 +195,7 @@ confirmation_dialog_response_cb (GtkDialog *dialog, gint response,
 	if (response != GTK_RESPONSE_OK)
 		return;
 
-	session = ephy_shell_get_session (ephy_shell);
+	session = SESSION (ephy_shell_get_session (ephy_shell));
 	windows = session_get_windows (session);
 
 	for (; windows != NULL; windows = windows->next)
@@ -918,7 +920,8 @@ provide_favicon (EphyNode *node, GValue *value, gpointer user_data)
 	const char *icon_location;
 	GdkPixbuf *pixbuf = NULL;
 
-	cache = ephy_embed_shell_get_favicon_cache (EPHY_EMBED_SHELL (ephy_shell));
+	cache = EPHY_FAVICON_CACHE
+		(ephy_embed_shell_get_favicon_cache (EPHY_EMBED_SHELL (ephy_shell)));
 	icon_location = ephy_node_get_property_string
 		(node, EPHY_NODE_PAGE_PROP_ICON);
 
@@ -930,7 +933,7 @@ provide_favicon (EphyNode *node, GValue *value, gpointer user_data)
 	}
 
 	g_value_init (value, GDK_TYPE_PIXBUF);
-	g_value_set_object (value, pixbuf);
+	g_value_set_object_take_ownership (value, pixbuf);
 }
 
 static void
@@ -1025,7 +1028,8 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 				   G_TYPE_STRING,
 				   EPHY_NODE_PAGE_PROP_TITLE,
 				   EPHY_NODE_PAGE_PROP_PRIORITY,
-				   EPHY_NODE_VIEW_AUTO_SORT,
+				   EPHY_NODE_VIEW_AUTO_SORT |
+				   EPHY_NODE_VIEW_SEARCHABLE,
 				   provide_favicon);
 	gtk_container_add (GTK_CONTAINER (scrolled_window), sites_view);
 	gtk_widget_show (sites_view);
@@ -1069,7 +1073,8 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 					   EPHY_NODE_PAGE_PROP_LOCATION);
 	col = ephy_node_view_add_column (EPHY_NODE_VIEW (pages_view), _("Title"),
 				         G_TYPE_STRING, EPHY_NODE_PAGE_PROP_TITLE,
-				         -1, EPHY_NODE_VIEW_USER_SORT, NULL);
+				         -1, EPHY_NODE_VIEW_USER_SORT |
+					 EPHY_NODE_VIEW_SEARCHABLE, NULL);
 	gtk_tree_view_column_set_max_width (col, 250);
 	col = ephy_node_view_add_column (EPHY_NODE_VIEW (pages_view), _("Address"),
 				         G_TYPE_STRING, EPHY_NODE_PAGE_PROP_LOCATION,
