@@ -34,7 +34,12 @@
 #include "ephy-bookmarks-editor.h"
 #include "ephy-history-window.h"
 #include "ephy-debug.h"
+#include "ephy-plugin.h"
 #include "toolbar.h"
+#include "session.h"
+#include "downloader-view.h"
+#include "ephy-toolbars-model.h"
+#include "ephy-autocompletion.h"
 
 #include <string.h>
 #include <libgnomeui/gnome-client.h>
@@ -43,6 +48,7 @@
 #include <gtk/gtksignal.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkmessagedialog.h>
+#include <dirent.h>
 
 #ifdef ENABLE_NAUTILUS_VIEW
 
@@ -61,6 +67,7 @@ struct EphyShellPrivate
 	EphyToolbarsModel *toolbars_model;
 	GtkWidget *bme;
 	GtkWidget *history_window;
+	GList *plugins;
 };
 
 static void
@@ -145,6 +152,39 @@ ephy_shell_new_window_cb (EphyEmbedShell *shell,
 	*new_embed = ephy_tab_get_embed (new_tab);
 }
 
+static void
+ephy_shell_load_plugins (EphyShell *es)
+{
+	DIR *d;
+	struct dirent *e;
+
+	d = opendir (PLUGINS_DIR);
+
+	if (d == NULL)
+	{
+		return;
+	}
+
+	while ((e = readdir (d)) != NULL)
+	{
+		char *plugin;
+
+		plugin = g_strconcat (PLUGINS_DIR"/", e->d_name, NULL);
+
+		if (g_str_has_suffix (plugin, ".so"))
+		{
+			EphyPlugin *obj;
+
+			obj = ephy_plugin_new (plugin);
+			es->priv->plugins = g_list_append
+				(es->priv->plugins, obj);
+		}
+
+		g_free (plugin);
+	}
+	closedir (d);
+}
+
 static void
 ephy_shell_init (EphyShell *gs)
 {
@@ -157,6 +197,7 @@ ephy_shell_init (EphyShell *gs)
 	gs->priv->bme = NULL;
 	gs->priv->history_window = NULL;
 	gs->priv->toolbars_model = NULL;
+	gs->priv->plugins = NULL;
 
 	ephy_shell = gs;
 	g_object_add_weak_pointer (G_OBJECT(ephy_shell),
@@ -170,25 +211,33 @@ ephy_shell_init (EphyShell *gs)
 
 	/* This ensures mozilla is fired up */
 	single = ephy_embed_shell_get_embed_single (EPHY_EMBED_SHELL (gs));
+	if (single != NULL)
+	{
+		g_signal_connect (G_OBJECT (single),
+				  "new_window_orphan",
+				  G_CALLBACK(ephy_shell_new_window_cb),
+				  NULL);
 
-	g_signal_connect (G_OBJECT (single),
-			  "new_window_orphan",
-			  G_CALLBACK(ephy_shell_new_window_cb),
-			  NULL);
+		ephy_init_services (gs);
+	}
+	else
+	{
+		GtkWidget *dialog;
 
-	ephy_init_services (gs);
-}
+		dialog = gtk_message_dialog_new
+			(NULL,
+                         GTK_DIALOG_MODAL,
+                         GTK_MESSAGE_ERROR,
+                         GTK_BUTTONS_CLOSE,
+                         _("Epiphany can't be used now. "
+                         "Mozilla initialization failed. Check your "
+                         "MOZILLA_FIVE_HOME environmental variable."));
+		gtk_dialog_run (GTK_DIALOG (dialog));
 
-static void
-save_toolbars (EggToolbarsModel *model)
-{
-	char *xml_file;
+		exit (0);
+	}
 
-	xml_file = g_build_filename (ephy_dot_dir (),
-                                     "ephy-toolbar.xml",
-                                     NULL);
-	egg_toolbars_model_save (model, xml_file);
-	g_free (xml_file);
+	ephy_shell_load_plugins (gs);
 }
 
 static void
@@ -205,10 +254,12 @@ ephy_shell_finalize (GObject *object)
 
 	g_assert (ephy_shell == NULL);
 
+	g_list_foreach (gs->priv->plugins, (GFunc)g_type_module_unuse, NULL);
+	g_list_free (gs->priv->plugins);
+
 	LOG ("Unref toolbars model")
 	if (gs->priv->toolbars_model)
 	{
-		save_toolbars (EGG_TOOLBARS_MODEL (gs->priv->toolbars_model));
 		g_object_unref (G_OBJECT (gs->priv->toolbars_model));
 	}
 
@@ -316,7 +367,7 @@ ephy_shell_get_active_window (EphyShell *gs)
 	Session *session;
 	const GList *windows;
 
-	session = ephy_shell_get_session (gs);
+	session = SESSION (ephy_shell_get_session (gs));
 	windows = session_get_windows (session);
 
 	if (windows == NULL) return NULL;
@@ -461,7 +512,7 @@ ephy_nautilus_view_new (BonoboGenericFactory *factory, const char *id,
  *
  * Return value: the current session.
  **/
-Session *
+GObject *
 ephy_shell_get_session (EphyShell *gs)
 {
 	if (!gs->priv->session)
@@ -472,10 +523,10 @@ ephy_shell_get_session (EphyShell *gs)
                          (gpointer *)&gs->priv->session);
 	}
 
-	return gs->priv->session;
+	return G_OBJECT (gs->priv->session);
 }
 
-EphyAutocompletion *
+GObject *
 ephy_shell_get_autocompletion (EphyShell *gs)
 {
 	EphyShellPrivate *p = gs->priv;
@@ -497,7 +548,7 @@ ephy_shell_get_autocompletion (EphyShell *gs)
 		ephy_autocompletion_add_source (p->autocompletion,
 						EPHY_AUTOCOMPLETION_SOURCE (bmk));
 	}
-	return p->autocompletion;
+	return G_OBJECT (p->autocompletion);
 }
 
 EphyBookmarks *
@@ -511,41 +562,22 @@ ephy_shell_get_bookmarks (EphyShell *gs)
 	return gs->priv->bookmarks;
 }
 
-EphyToolbarsModel *
+GObject *
 ephy_shell_get_toolbars_model (EphyShell *gs)
 {
 	if (gs->priv->toolbars_model == NULL)
 	{
-		char *xml_file;
-		EggToolbarsModel *model;
 		EphyBookmarks *bookmarks;
 
 		bookmarks = ephy_shell_get_bookmarks (gs);
 
 		gs->priv->toolbars_model = ephy_toolbars_model_new (bookmarks);
-		model = EGG_TOOLBARS_MODEL (gs->priv->toolbars_model);
-
-		xml_file = g_build_filename (ephy_dot_dir (),
-		                             "ephy-toolbar.xml",
-		                             NULL);
-		if (g_file_test (xml_file, G_FILE_TEST_EXISTS))
-		{
-			egg_toolbars_model_load (model, xml_file);
-		}
-		else
-		{
-			const char *default_xml;
-
-			default_xml = ephy_file ("epiphany-toolbar.xml");
-			egg_toolbars_model_load (model, default_xml);
-		}
-		g_free (xml_file);
 
 		g_object_set (bookmarks, "toolbars_model",
 			      gs->priv->toolbars_model, NULL);
 	}
 
-	return gs->priv->toolbars_model;
+	return G_OBJECT (gs->priv->toolbars_model);
 }
 
 static void
