@@ -42,8 +42,10 @@
 #include "ephy-new-bookmark.h"
 #include "ephy-stock-icons.h"
 #include "eggtoolbar.h"
+#include "ephy-toolbars-model.h"
 
 #include <string.h>
+#include <bonobo/bonobo-i18n.h>
 
 static void toolbar_class_init (ToolbarClass *klass);
 static void toolbar_init (Toolbar *t);
@@ -119,12 +121,7 @@ toolbar_get_type (void)
 static void
 go_location_cb (EggAction *action, char *location, EphyWindow *window)
 {
-	EphyEmbed *embed;
-
-	embed = ephy_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-
-	ephy_embed_load_url (embed, location);
+	ephy_window_load_url (window, location);
 }
 
 static void
@@ -142,10 +139,11 @@ topic_destroy_cb (EphyNode *node,
 	EphyToolbarsModel *model;
 	long id;
 
-	model = ephy_shell_get_toolbars_model (ephy_shell);
+	model = EPHY_TOOLBARS_MODEL
+		(ephy_shell_get_toolbars_model (ephy_shell));
 
 	id = ephy_node_get_id (node);
-	name = g_strdup_printf ("GoTopicId%ld", ephy_node_get_id (node));
+	name = ephy_toolbars_model_get_action_name (model, TRUE, id);
 	action = egg_action_group_get_action (t->priv->action_group, name);
 	if (action)
 	{
@@ -165,10 +163,11 @@ bookmark_destroy_cb (EphyNode *node,
 	EphyToolbarsModel *model;
 	long id;
 
-	model = ephy_shell_get_toolbars_model (ephy_shell);
+	model = EPHY_TOOLBARS_MODEL
+		(ephy_shell_get_toolbars_model (ephy_shell));
 
 	id = ephy_node_get_id (node);
-	name = g_strdup_printf ("GoBookmarkId%ld", id);
+	name = ephy_toolbars_model_get_action_name (model, FALSE, id);
 	action = egg_action_group_get_action (t->priv->action_group, name);
 	if (action)
 	{
@@ -184,47 +183,67 @@ toolbar_ensure_action (Toolbar *t,
 		       const char *name)
 {
 	EggAction *action = NULL;
-	long id = 0;
 	EphyBookmarks *bookmarks;
 
 	LOG ("Ensure action %s", name)
 
 	bookmarks = ephy_shell_get_bookmarks (ephy_shell);
 
-	if (g_str_has_prefix (name, "GoBookmarkId"))
+	if (g_str_has_prefix (name, "GoBookmark-"))
 	{
 		EphyNode *node;
 
-		if (!ephy_string_to_int (name + strlen ("GoBookmarkId"), &id))
-		{
-			return;
-		}
-
 		LOG ("Create action %s", name)
 
-		action = ephy_bookmark_action_new (name, id);
+		node = ephy_bookmarks_find_bookmark
+			(bookmarks, name + strlen ("GoBookmark-"));
+		g_return_if_fail (node != NULL);
+
+		action = ephy_bookmark_action_new (name, ephy_node_get_id (node));
 		g_signal_connect (action, "go_location",
 				  G_CALLBACK (go_location_cb), t->priv->window);
 		egg_action_group_add_action (t->priv->action_group, action);
 		g_object_unref (action);
 
-		node = ephy_bookmarks_get_from_id (bookmarks, id);
 		ephy_node_signal_connect_object (node,
 					         EPHY_NODE_DESTROY,
 					         (EphyNodeCallback) bookmark_destroy_cb,
 					         G_OBJECT (t));
 	}
-	else if (g_str_has_prefix (name, "GoTopicId"))
+	else if (g_str_has_prefix (name, "GoTopic-"))
 	{
 		EphyNode *node;
 
-		if (!ephy_string_to_int (name + strlen ("GoTopicId"), &id))
+		LOG ("Create action %s", name)
+
+		node = ephy_bookmarks_find_keyword
+			(bookmarks, name + strlen ("GoTopic-"), FALSE);
+		g_return_if_fail (node != NULL);
+
+		action = ephy_topic_action_new (name, ephy_node_get_id (node));
+		g_signal_connect (action, "go_location",
+				  G_CALLBACK (go_location_cb),
+				  t->priv->window);
+		egg_action_group_add_action (t->priv->action_group, action);
+		g_object_unref (action);
+
+		ephy_node_signal_connect_object (node,
+					         EPHY_NODE_DESTROY,
+					         (EphyNodeCallback) topic_destroy_cb,
+					         G_OBJECT (t));
+	}
+	else if (g_str_has_prefix (name, "GoSpecialTopic-"))
+	{
+		EphyNode *node;
+		long id;
+
+		LOG ("Create action %s", name)
+
+		if (!ephy_string_to_int (name + strlen ("GoSpecialTopic-"), &id))
 		{
 			return;
 		}
 
-		LOG ("Create action %s", name)
-
 		action = ephy_topic_action_new (name, id);
 		g_signal_connect (action, "go_location",
 				  G_CALLBACK (go_location_cb),
@@ -305,6 +324,7 @@ toolbar_setup_actions (Toolbar *t)
 			       "name", "NavigationBack",
 			       "label", _("Back"),
 			       "stock_id", GTK_STOCK_GO_BACK,
+			       "tooltip", _("Go back"),
 			       "window", t->priv->window,
 			       "direction", EPHY_NAVIGATION_DIRECTION_BACK,
 			       NULL);
@@ -317,6 +337,7 @@ toolbar_setup_actions (Toolbar *t)
 			       "name", "NavigationForward",
 			       "label", _("Forward"),
 			       "stock_id", GTK_STOCK_GO_FORWARD,
+			       "tooltip", _("Go forward"),
 			       "window", t->priv->window,
 			       "direction", EPHY_NAVIGATION_DIRECTION_FORWARD,
 			       NULL);
@@ -328,9 +349,10 @@ toolbar_setup_actions (Toolbar *t)
 	action = g_object_new (EPHY_TYPE_NAVIGATION_ACTION,
 			       "name", "NavigationUp",
 			       "label", _("Up"),
+			       "stock_id", GTK_STOCK_GO_UP,
+			       "tooltip", _("Go up"),
 			       "window", t->priv->window,
 			       "direction", EPHY_NAVIGATION_DIRECTION_UP,
-			       "stock_id", GTK_STOCK_GO_UP,
 			       NULL);
 	g_signal_connect (action, "activate",
 			  G_CALLBACK (window_cmd_go_up), t->priv->window);
@@ -351,6 +373,7 @@ toolbar_setup_actions (Toolbar *t)
 			       "name", "Location",
 			       "label", _("Address Entry"),
 			       "stock_id", EPHY_STOCK_ENTRY,
+			       "tooltip", _("Enter a web address to open, or a phrase to search for on the web"),
 			       NULL);
 	g_signal_connect (action, "go_location",
 			  G_CALLBACK (go_location_cb), t->priv->window);
@@ -360,6 +383,7 @@ toolbar_setup_actions (Toolbar *t)
 	action = g_object_new (EPHY_TYPE_ZOOM_ACTION,
 			       "name", "Zoom",
 			       "label", _("Zoom"),
+			       "tooltip", _("Adjust the text size"),
 			       "zoom", 1.0,
 			       NULL);
 	g_signal_connect (action, "zoom_to_level",
@@ -379,6 +403,7 @@ toolbar_setup_actions (Toolbar *t)
 			       "name", "ToolbarGo",
 			       "label", _("Go"),
 			       "stock_id", GTK_STOCK_JUMP_TO,
+			       "tooltip", _("Go to the address entered in the address entry"),
 			       NULL);
 	g_signal_connect (action, "activate",
 			  G_CALLBACK (window_cmd_load_location), t->priv->window);
@@ -400,7 +425,8 @@ init_bookmarks_toolbar (Toolbar *t)
 	EphyToolbarsModel *model;
 	int i, n_toolbars;
 
-	model = ephy_shell_get_toolbars_model (ephy_shell);
+	model = EPHY_TOOLBARS_MODEL
+		(ephy_shell_get_toolbars_model (ephy_shell));
 	n_toolbars = egg_toolbars_model_n_toolbars
 		(EGG_TOOLBARS_MODEL (model));
 
@@ -505,7 +531,8 @@ toolbar_set_window (Toolbar *t, EphyWindow *window)
 			  G_CALLBACK (action_request_cb),
 			  NULL);
 
-	model = ephy_shell_get_toolbars_model (ephy_shell);
+	model = EPHY_TOOLBARS_MODEL
+		(ephy_shell_get_toolbars_model (ephy_shell));
 	g_signal_connect (EGG_TOOLBARS_MODEL (model), "toolbar_added",
 			  G_CALLBACK (update_toolbar_remove_flag),
 			  NULL);
@@ -707,7 +734,8 @@ toolbar_set_visibility (Toolbar *t,
 	EphyToolbarsModel *model;
 	int i, n_toolbars;
 
-	model = ephy_shell_get_toolbars_model (ephy_shell);
+	model = EPHY_TOOLBARS_MODEL
+		(ephy_shell_get_toolbars_model (ephy_shell));
 	n_toolbars = egg_toolbars_model_n_toolbars
 		(EGG_TOOLBARS_MODEL (model));
 
